# Copyright (C) 2020 Alteryx, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Class that saves and retrieves AMP IO information."""
import logging
import uuid
from pathlib import Path
from typing import Any, cast

from ayx_plugin_sdk.core.exceptions import WorkflowRuntimeError
from ayx_plugin_sdk.providers.amp_provider.repositories.environment_repository import (
    EnvironmentRepository,
)
from ayx_plugin_sdk.providers.amp_provider.repositories.grpc_repository import (
    GrpcRepository,
)
from ayx_plugin_sdk.providers.amp_provider.repositories.singleton import Singleton
from ayx_plugin_sdk.providers.amp_provider.resources.generated.output_message_data_pb2 import (
    OutputMessageData,
    OutputMessageTypes,
)
from ayx_plugin_sdk.providers.amp_provider.resources.generated.password_data_pb2 import (
    PasswordData,
)
from ayx_plugin_sdk.providers.amp_provider.resources.generated.translate_message_data_pb2 import (
    TranslateMessageData,
)

logger = logging.getLogger(__name__)


class IORepository(metaclass=Singleton):
    """Repository that stores IO information."""

    def __init__(self) -> None:
        """Initialize the IO repository."""

    @staticmethod
    def save_error(error_msg: str) -> None:
        """Display an error message in the Results window."""
        GrpcRepository().get_sdk_engine_client().OutputMessage(
            OutputMessageData(message_type=OutputMessageTypes.OMT_Error, message=error_msg)  # type: ignore
        )

    @staticmethod
    def save_warn(warn_msg: str) -> None:
        """Display a warning message in the Results window."""
        GrpcRepository().get_sdk_engine_client().OutputMessage(
            OutputMessageData(message_type=OutputMessageTypes.OMT_Warning, message=warn_msg)  # type: ignore
        )

    @staticmethod
    def save_info(info_msg: str) -> None:
        """Display an info message in the Results window."""
        GrpcRepository().get_sdk_engine_client().OutputMessage(
            OutputMessageData(message_type=OutputMessageTypes.OMT_Info, message=info_msg)  # type: ignore
        )

    @staticmethod
    def get_translate_msg(msg: str, *args: Any) -> str:
        """Translate a message to the current locale."""
        interpolation_data = [str(arg) for arg in args] if len(args) > 0 else []
        res = (
            GrpcRepository()
            .get_sdk_engine_client()
            .TranslateMessage(
                TranslateMessageData(
                    unlocalized_string=msg, interpolation_data=interpolation_data
                )
            )
        )

        return cast(str, res.translated_message)

    @staticmethod
    def update_progress(percent: float) -> None:
        """Update tool progress."""
        # TODO

    @staticmethod
    def get_temp_file(extension: str = "tmp") -> "Path":
        """Create a temporary file managed by Designer."""
        temp_file_name = f"temp-file-{str(uuid.uuid4())}.{str(extension)}"
        engine_temp_dir = EnvironmentRepository().get_temp_dir()
        temp_file_path = Path(engine_temp_dir) / (temp_file_name)
        try:
            temp_file_path.touch()
        except FileNotFoundError:
            # path does not exist
            logger.error(f"Engine.TempFilePath ({engine_temp_dir}) does not exist")
        except IOError:
            # path exists but no write permissions
            logger.error(f"No write permissions for directory {engine_temp_dir}")

        return Path(temp_file_path)

    @staticmethod
    def decrypt_password(password: str) -> str:  # noqa: D102
        """
        Decrypt password.

        Parameters
        ----------
        password
            The password to decrypt.

        Returns
        -------
        str
            The decrypted password.
        """
        import grpc

        try:
            decrypted = (
                GrpcRepository()
                .get_sdk_engine_client()
                .DecryptPassword(PasswordData(password=password))
            )
        except grpc.RpcError:
            raise WorkflowRuntimeError("Error decrypting password")
        return cast(str, decrypted.password)
